
from __future__ import unicode_literals

__version__ = "HEAD"
__project__ = "aiorqlite"

__author__ = "Terra Brown"
__email__ = "superloach42@gmail.com"

__copyright__ = None
__license__ = "MIT"
__description__ = "asynchronous Python client for rqlite"

UNLIMITED_REDIRECTS = -1
